/*
 * PmodALS.c
 *
 *  Created on: Nov 6, 2017
 *      Author: Krzysztof Chojnowski
 */

#include "stm32l4xx_hal.h"
#include "PmodALS.h"

SPI_HandleTypeDef pmodAlsSpi;

void PmodALS_Config(void)
{
  // Configure the SPI connected to the Pmod module. Only the MISO line is required.
  pmodAlsSpi.Instance = SPI1;
  pmodAlsSpi.Init.Mode = SPI_MODE_MASTER;
  pmodAlsSpi.Init.Direction = SPI_DIRECTION_2LINES_RXONLY;
  pmodAlsSpi.Init.DataSize = SPI_DATASIZE_16BIT;
  pmodAlsSpi.Init.CLKPolarity = SPI_POLARITY_HIGH;
  pmodAlsSpi.Init.CLKPhase = SPI_PHASE_1EDGE;
  pmodAlsSpi.Init.NSS = SPI_NSS_SOFT;
  pmodAlsSpi.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_64;
  pmodAlsSpi.Init.FirstBit = SPI_FIRSTBIT_MSB;
  pmodAlsSpi.Init.TIMode = SPI_TIMODE_DISABLE;
  pmodAlsSpi.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
  pmodAlsSpi.Init.NSSPMode = SPI_NSS_PULSE_DISABLE;

  HAL_SPI_Init(&pmodAlsSpi);
}

void HAL_SPI_MspInit(SPI_HandleTypeDef *hspi)
{
  // Initialize GPIO used by the SPI1 peripheral. The CS is controlled by the software (PB0 pin).
  __HAL_RCC_SPI1_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();
  __HAL_RCC_GPIOE_CLK_ENABLE();

  GPIO_InitTypeDef GPIO_InitStruct;

  GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
  GPIO_InitStruct.Pull = GPIO_PULLDOWN;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.Alternate = GPIO_AF5_SPI1;
  GPIO_InitStruct.Pin = GPIO_PIN_1;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  GPIO_InitStruct.Pin = GPIO_PIN_14;
  HAL_GPIO_Init(GPIOE, &GPIO_InitStruct);

  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_PULLDOWN;
  GPIO_InitStruct.Pin = GPIO_PIN_0;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);
  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_SET);
}

uint8_t PmodALS_GetValue(void)
{
  uint16_t value = 0;

  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_RESET);
  HAL_SPI_Receive(&pmodAlsSpi, (uint8_t*)&value, 1, 100);
  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_SET);

  // The data obtained from the ADC is 16 bit, but the light level value is only 8 bit.
  // It is shifted left by 4 bits.
  return (value >> 4);
}
