/*
 * pwm.c
 *
 *  Created on: Nov 6, 2017
 *      Author: Krzysztof Chojnowski
 */

#include "stm32l4xx_hal.h"
#include "pwm.h"

TIM_HandleTypeDef pwmHandle;

void Pwm_Config(void)
{
  // Configure and enable the TIM4 channel 1 PWM output. It is connected to LED-RGB blue (PD12)
  // on the Kameleon board. The PWM period is 100 ticks and the starting duty cycle
  // is set to 0%.
  pwmHandle.Instance = TIM4;
  pwmHandle.Init.Prescaler = 100;
  pwmHandle.Init.CounterMode = TIM_COUNTERMODE_UP;
  pwmHandle.Init.Period = 100;
  pwmHandle.Init.ClockDivision = TIM_CLOCKDIVISION_DIV4;
  pwmHandle.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_ENABLE;

  TIM_OC_InitTypeDef pwmChannel;
  pwmChannel.OCMode = TIM_OCMODE_PWM1;
  pwmChannel.Pulse = 0;
  pwmChannel.OCPolarity = TIM_OCPOLARITY_HIGH;
  pwmChannel.OCFastMode = TIM_OCFAST_DISABLE;

  HAL_TIM_PWM_Init(&pwmHandle);
  HAL_TIM_PWM_ConfigChannel(&pwmHandle, &pwmChannel, TIM_CHANNEL_1);

  HAL_TIM_Base_Start(&pwmHandle);
  HAL_TIM_PWM_Start(&pwmHandle, TIM_CHANNEL_1);
}

void HAL_TIM_PWM_MspInit(TIM_HandleTypeDef *htim)
{
  // Configure the PD12 connected to LED-RGB blue as alternate function - PWM output.
  __HAL_RCC_TIM4_CLK_ENABLE();
  __HAL_RCC_GPIOD_CLK_ENABLE();

  GPIO_InitTypeDef GPIO_InitStruct;
  GPIO_InitStruct.Mode  = GPIO_MODE_AF_PP;
  GPIO_InitStruct.Pull  = GPIO_PULLDOWN;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.Alternate = GPIO_AF2_TIM4;
  GPIO_InitStruct.Pin = GPIO_PIN_12;
  HAL_GPIO_Init(GPIOD, &GPIO_InitStruct);
}

void Pwm_SetDuty(uint8_t duty)
{
  if(duty > 100)
    return;

  // Change the duty cycle of the configured PWM channel.
  __HAL_TIM_SET_COMPARE(&pwmHandle, TIM_CHANNEL_1, duty);
}

